using System;
using System.Security.Cryptography;
using System.Text;
using System.IO;

namespace Example
{
    /// <summary>
    /// Verify a secret on a SmartDongle.
    ///
    ///
    /// MicroWorks, Inc.
    /// 2808 North Cole Road
    /// Boise, ID 83704
    ///
    /// http://www.smartdongle.com
    ///
    /// </summary>
    class Example3b
    {
        [STAThread]

        static void Main(string[] args)
        {
            try
            {
                UInt64 P1, P2;
                SmartDongle.SmartDnglError error;
                System.Text.ASCIIEncoding  encoding = new System.Text.ASCIIEncoding();

                /*
                 *  Padding is zeroes and CipherMode has been set to
                 *  Electronic Codebook (ECB) to match the algorithm in the C example.
                 *
                 *  The AES key and user secret below are for example only, the user
                 *  should create a new key and secret for their application.
                 *
                 *  A System.Security.Cryptography.CryptographicException will
                 *  occur if illegal AES key length or AES key size are given below.
                */
                const string aes_key = "7obdgEn2'TH>]SE|";
                const int aes_KeySize = 128;   // Size in bits
                const int aes_BlockSize = 128; // Size in bits
                const string your_secret = "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc";

                Rijndael RijndaelAlg = Rijndael.Create();
                RijndaelAlg.KeySize = aes_KeySize;
                RijndaelAlg.BlockSize = aes_BlockSize;
                RijndaelAlg.Mode = CipherMode.ECB;
                RijndaelAlg.Padding = PaddingMode.None;

                byte[] rgbKey = encoding.GetBytes(aes_key);
                byte[] rgbIV = rgbKey;

                ICryptoTransform decrypt = RijndaelAlg.CreateDecryptor(rgbKey, rgbIV);

                /* Your SmartDongle keys go here */
                P1 = 0xec6cc589aefd1e75;
                P2 = 0xfcec0a6a82747b3f;

                /*
                 *  Read the encrypted secret from the SmartDongle,
                 *  decrypt and compare with original secret.
                 */
                byte[] secret = Encoding.UTF8.GetBytes(your_secret);

                /*
                 * Cipher length is a multiple of AES blocksize.
		 * Note that AES blocksize is in bits.
                 */
                int cipherBlocks = secret.Length / (aes_BlockSize / 8);
                if (secret.Length % (aes_BlockSize / 8) > 0) {
                    cipherBlocks++;
                }

                byte[] cipherText = new byte[cipherBlocks * (aes_BlockSize / 8)];
                UInt16 address = 0;
                error = SmartDongle.Read(P1, P2, address, cipherText, cipherText.Length);
                if (error != SmartDongle.SmartDnglError.UskOk) {

                    System.Console.WriteLine(SmartDongle.GetErrorString(error));

                    // handle error
                    return;
                }

                /* Decrypt */
                byte[] result = decrypt.TransformFinalBlock(cipherText, 0, cipherText.Length);

                /* Compare to original secret, using expected orignal secret length */
                int i;
                for (i = 0; (i < secret.Length) && (result[i] == secret[i]); i++) ;
                if (i == secret.Length)
                {
                    System.Console.WriteLine("Success: Found Valid Secret\n");
                }
                else
                {
                    System.Console.WriteLine("Failed: Invalid Secret\n");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error: {0}", e.Message);
            }
            System.Console.WriteLine("Press <Enter> to Exit");
            System.Console.ReadLine();
        }
    }
}

